#property script_show_inputs

input datetime inFromDate = D'2020.01.01'; // From which date to take data
input bool inReverse = false;              // ReverseTime

#include <fxsaber\HistoryTicks\ArrayResize.mqh> // https://www.mql5.com/ru/code/20298
#include <fxsaber\ThirdPartyTicks\CustomSymbol.mqh> // https://www.mql5.com/ru/code/20225

#define HOUR 3600
#define DAY (24 * HOUR)
#define WEEK 7

int TimeDayOfWeek(const datetime Date)
  {
   MqlDateTime mTime;

   ::TimeToStruct(Date,mTime);

   return(mTime.day_of_week);
  }

// https://www.mql5.com/ru/forum/170953/page8#comment_6940794
datetime GetTimeDayOfWeek( const datetime TimeSource, const int Shift = 0, const ENUM_DAY_OF_WEEK Day = SUNDAY )
{
  const datetime Res = TimeSource / DAY * DAY;

  return(Res - (((WEEK + (TimeDayOfWeek(Res) - Day)) % WEEK) + Shift * WEEK) * DAY);
}

void ReverseTick( MqlTick &Tick, const long &Offset )
{
  Tick.time_msc = Offset - Tick.time_msc;
  Tick.time = (datetime)(Tick.time_msc / 1000);

  return;
}

void PrintHours( void )
{
  for (datetime time = 0; time < DAY; time += HOUR)
    Print(TimeToString(time, TIME_MINUTES) + " -> " + TimeToString(DAY - time, TIME_MINUTES));

  return;
}

void ReverseTicks( MqlTick &Ticks[] )
{
  const int Size = ArraySize(Ticks);

  if (Size)
  {
    const long Offset = (long)(GetTimeDayOfWeek(Ticks[0].time, 0, MONDAY) + GetTimeDayOfWeek(Ticks[Size - 1].time, -1, SATURDAY)) * 1000;

    for (int i = 0; i < Size; i++)
      ReverseTick(Ticks[i], Offset);

    PrintHours();

    ArrayReverse(Ticks);
  }

  return;
}

int Filter( MqlTick &Ticks[] )
{
  return(ArraySize(Ticks));
}

bool Symbol2Custom( CUSTOMSYMBOL &CustomSymbol, const datetime FromDate = 0, const datetime Reverse = false )
{
  MqlTick Ticks[];

  const int BeforeFilter = CopyTicksRange(_Symbol, Ticks, COPY_TICKS_ALL, (ulong)FromDate * 1000);

  if (Reverse)
    ReverseTicks(Ticks);

  const int AfterFilter = Filter(Ticks);

  Print("After Filter Ticks = " + (string)AfterFilter +
          (BeforeFilter ? " (" + ::DoubleToString(100.0 * AfterFilter / BeforeFilter, 2) + "%)" : NULL));

//  CustomSymbol.AddTicks(Ticks);
  CustomSymbol.SwapTicks(Ticks);

  const int Amount = _CS(true) ? CustomSymbol.DataToSymbol() : -1;
  const bool Res = (Amount > 0);

  if (!Res)
  {
    Print(CustomSymbol.Name + " error write ticks - " + (_CS(true) ? (string)::GetLastError() : "Stopped!"));

    CustomSymbol.Delete();
  }
  else if (CustomSymbol.On())
  {
    Print(CustomSymbol.Name + " saved ticks = " + (string)Amount);

    // https://www.mql5.com/ru/forum/170952/page140#comment_12489183
    CustomSymbol.SetProperty(SYMBOL_VOLUME_STEP, 0.01);
    CustomSymbol.SetProperty(SYMBOL_VOLUME_MAX, 1e4);
    CustomSymbol.SetProperty(SYMBOL_VOLUME_MIN, 0.01);

    CustomSymbol.SetProperty(SYMBOL_TRADE_STOPS_LEVEL, 0);

    ChartOpen(CustomSymbol.Name, PERIOD_M1);
  }

  return(Res);
}

void OnStart()
{
  CUSTOMSYMBOL CustomSymbol("CUSTOM" +(inReverse ? "R" : NULL) + "_" + _Symbol);

  if (!CustomSymbol.IsExist() || !Symbol2Custom(CustomSymbol, /*SymbolInfoInteger(_Symbol, SYMBOL_CUSTOM) ? 0 : */inFromDate, inReverse))
    Alert("Error with Symbol " + CustomSymbol.Name + " - " + (string)GetLastError());
}